package com.pdaaphm.system.service.impl;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import com.pdaaphm.biz.domain.BaseResponse;
import com.pdaaphm.common.config.PadaphmConfig;
import com.pdaaphm.common.utils.DateUtils;
import com.pdaaphm.common.utils.DictUtils;
import com.pdaaphm.common.utils.StringUtils;
import com.pdaaphm.common.utils.file.FileUtils;
import com.pdaaphm.common.utils.uuid.Seq;
import com.pdaaphm.system.domain.AlgConfig;
import com.pdaaphm.system.domain.Data;
import com.pdaaphm.system.service.IAlgConfigService;
import com.pdaaphm.system.service.IDataService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.http.MediaType;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import com.pdaaphm.system.mapper.TDataProcessMapper;
import com.pdaaphm.system.domain.TDataProcess;
import com.pdaaphm.system.service.ITDataProcessService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

/**
 * 数据处理Service业务层处理
 *
 * @author ruoyi
 * @date 2024-08-20
 */
@Service
public class TDataProcessServiceImpl implements ITDataProcessService {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private TDataProcessMapper tDataProcessMapper;

    @Autowired
    IAlgConfigService algConfigService;

    @Autowired
    IDataService dataService;

    /**
     * 查询数据处理
     *
     * @param id 数据处理主键
     * @return 数据处理
     */
    @Override
    public TDataProcess selectTDataProcessById(Long id) {
        return tDataProcessMapper.selectTDataProcessById(id);
    }

    /**
     * 查询数据处理列表
     *
     * @param tDataProcess 数据处理
     * @return 数据处理
     */
    @Override
    public List<TDataProcess> selectTDataProcessList(TDataProcess tDataProcess) {
        return tDataProcessMapper.selectTDataProcessList(tDataProcess);
    }

    @Override
    public List<TDataProcess> selectPreprocessDataList(TDataProcess tDataProcess) {
        return tDataProcessMapper.selectPreprocessDataList(tDataProcess);
    }

    /**
     * 新增数据处理
     *
     * @param tDataProcess 数据处理
     * @return 结果
     */
    @Override
    public int insertTDataProcess(TDataProcess tDataProcess) {
        tDataProcess.setCreateTime(DateUtils.getNowDate());
        return tDataProcessMapper.insertTDataProcess(tDataProcess);
    }

    /**
     * 修改数据处理
     *
     * @param tDataProcess 数据处理
     * @return 结果
     */
    @Override
    public int updateTDataProcess(TDataProcess tDataProcess) {
        tDataProcess.setUpdateTime(DateUtils.getNowDate());
        return tDataProcessMapper.updateTDataProcess(tDataProcess);
    }

    /**
     * 批量删除数据处理
     *
     * @param ids 需要删除的数据处理主键
     * @return 结果
     */
    @Override
    public int deleteTDataProcessByIds(Long[] ids) {
        return tDataProcessMapper.deleteTDataProcessByIds(ids);
    }

    /**
     * 删除数据处理信息
     *
     * @param id 数据处理主键
     * @return 结果
     */
    @Override
    public int deleteTDataProcessById(Long id) {
        return tDataProcessMapper.deleteTDataProcessById(id);
    }

    @Override
    public List<TDataProcess> getDataProcessByType(String type) {
        TDataProcess dataProcess = new TDataProcess();
        dataProcess.setProcessType(type);
        return this.selectTDataProcessList(dataProcess);
    }

    @Override
    public String runDataProcess(Long id) {
        String errorMsg = "";
        TDataProcess dataProcess = this.selectTDataProcessById(id);
        if (dataProcess != null) {
            AlgConfig algConfig = algConfigService.selectAlgConfigById(dataProcess.getProcessAlgId());
            Data data = dataService.selectDataById(dataProcess.getProcessedDataId());
            String bizAlgType = DictUtils.getDictLabel("biz_alg_type", dataProcess.getProcessType());

            if (algConfig == null || data == null || bizAlgType == null) {
                logger.warn("无法查找到该DataProcess的数据，id:{},algConfig:{},data:{},bizAlgType:{}", id, algConfig, data, bizAlgType);
                errorMsg = "数据有残缺";
            } else {
                // 异步调用
                ITDataProcessService proxy = applicationContext.getBean(ITDataProcessService.class);
                proxy.asyncRunDataProcess(dataProcess, algConfig, data, bizAlgType);
            }
        } else {
            logger.warn("无法查找到该DataProcess的数据，id:{}", id);
            errorMsg = "未找到该数据!";
        }
        return errorMsg;
    }

    @Async
    @Transactional
    public void asyncRunDataProcess(TDataProcess dataProcess, AlgConfig algConfig, Data data, String bizAlgType) {
        // 状态设置为处理中
        dataProcess.setProcessStatus("2");
        dataProcess.setStartTime(new Date());
        dataProcess.setEndTime(null);
        this.updateTDataProcess(dataProcess);
        Map<String, String> resultMap = doRunDataProcess(algConfig, data, bizAlgType);
        if (resultMap.get("errorMsg") == null) {
            // 成功场景
            // create new data
            Data newData = new Data();
            String resultPath = resultMap.get("resultPath");
            String resultImagePath = resultMap.get("resultImagePath");
            newData.setDataPath(resultPath.replaceFirst(PadaphmConfig.getProfile(),"/profile"));
            if (checkFileExist(resultImagePath)) {
                newData.setImagePath(resultImagePath.replaceFirst(PadaphmConfig.getProfile(),"/profile"));
            }
            newData.setDataType((Integer.parseInt(dataProcess.getProcessType()) + 1) + "");
            newData.setDataName(FileUtils.getNameNotSuffix(resultPath));
            dataService.insertData(newData);
            // update dataProcess
            dataProcess.setProcessStatus("3"); // 已完成
            dataProcess.setResultDataId(newData.getId()); // 已完成
            dataProcess.setEndTime(new Date());
            this.updateTDataProcess(dataProcess);
        } else {
            logger.error("dataProcess error,id:{},error:{}",dataProcess.getId(),resultMap.get("errorMsg"));
            // 失败场景
            dataProcess.setProcessStatus("4"); // 失败
            dataProcess.setEndTime(new Date());
            dataProcess.setLog(resultMap.get("errorMsg"));
            this.updateTDataProcess(dataProcess);
        }
    }

    private Map<String, String> doRunDataProcess(AlgConfig algConfig, Data data, String bizAlgType) {
        Map<String, String> result = new HashMap<>(2);
        String algUrl = algConfig.getAlgUrl();
        String dataPath = data.getDataPath();
        logger.info("doRunDataProcess.dataPath: {}", dataPath);
        String fileName = FileUtils.getNameNotSuffix(dataPath);
        logger.info("doRunDataProcess.fileName: {}", fileName);
        String originalFileName = fileName.split("_")[0];
        // 文件名 原始文件名_类型_时间戳
        String resultPath = StringUtils.format("{}/{}/{}_{}_{}.csv", PadaphmConfig.getUploadPath(), DateUtils.datePath(), originalFileName, bizAlgType, Seq.getId(Seq.uploadSeqType));
        String resultImagePath = StringUtils.format("{}/{}/{}_{}_{}.jpeg", PadaphmConfig.getUploadPath(), DateUtils.datePath(), originalFileName, bizAlgType, Seq.getId(Seq.uploadSeqType));

        String errorMsg = sendHttp(algUrl, dataPath.replace("/profile",PadaphmConfig.getProfile()), resultPath, resultImagePath);
        result.put("resultPath", resultPath);
        result.put("resultImagePath", resultImagePath);
        result.put("errorMsg", errorMsg);
        return result;
    }

    private boolean checkFileExist(String resultImagePath) {
        String realPath = resultImagePath;

        if (realPath.startsWith("/profile")) {
            realPath = realPath.replace("/profile", PadaphmConfig.getProfile());
        }

        Path path = Paths.get(realPath);
        return Files.exists(path);
    }

    private String sendHttp(String algUrl, String dataPath, String resultPath, String resultImagePath) {
        HttpComponentsClientHttpRequestFactory httpRequestFactory = new HttpComponentsClientHttpRequestFactory();
        httpRequestFactory.setConnectionRequestTimeout(3000);
        httpRequestFactory.setConnectTimeout(3000);
        httpRequestFactory.setReadTimeout(3000);
        WebClient webClient = WebClient.create(algUrl);

        Map<String, String> requestDto = new HashMap<>(4);
        requestDto.put("file_url", dataPath);
        requestDto.put("result_file_path", resultPath);
        requestDto.put("result_img_path", resultImagePath);

        // send post
        Mono<BaseResponse> result = webClient.post() // 使用POST方法
//                .uri("/request")  // 指定URI
                .contentType(MediaType.APPLICATION_JSON)  // 设置Content-Type为application/json
                .bodyValue(requestDto)  // 设置请求body
                .retrieve()  // 执行请求
                .bodyToMono(BaseResponse.class)
                .onErrorResume(e -> {
                    logger.error("http request error!!!", e);
                    if (e instanceof TimeoutException) {
                        // 对超时错误进行处理
                        return Mono.just(BaseResponse.error("Timeout error"));
                    } else {
                        // 对其他错误进行处理
                        return Mono.just(BaseResponse.error(e.getMessage()));
                    }
                });  // 将响应体转换为String
        // get response
        BaseResponse res = result.block();
        if (res == null) {
            logger.error("response is null");
            return "response is null";
        }
        if (!Integer.valueOf("200").equals(res.getCode())) {
            return res.getMsg();
        }
        return null;
    }
}
